# APA 6th Ref2RIS Converter v. 1.0
# Copyright (C) 2011 Deborah Fitchett
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
########################################################################

# For best results, prepare input according to the ReadMe instructions.
# Should be fairly good at parsing journal articles, books and chapters,
# reports, conference papers, and theses, except:
#  * Performs badly with italicised species names etc

# Tidy up tags, punctuation, spurious spacing, blank lines, and urls
s/\<[^/>]*\>$//
s/^\<\/[^>]*\>//
s/\(\<[^/>]*\>\)\([^0-9A-Za-z<>]*\)/\2\1/g
s/\([^0-9A-Za-z<>]*\)\(\<\/[^>]*\>\)/\2\1/g
s/\<\([^/>]*\)\>\([^a-zA-Z0-9<>]*\)\<\/\1\>/\2/g
s/\<\/\([^>]*\)\>\([^a-zA-Z0-9<>]*\)\<\1\>/\2/g
s/ *- */-/g
s/\( \)\([ ,]*\)/\1/g
s/^[ 	]*//
s/[ 	]*$//
s/[ 	][ 	]*/ /g
/^$/d
s/\([^/]\)www/\1http:\/\/www/g

# Deliminate records (adding a copy of the full record to the notes field)
s/\(.*\)$/\1@@N1  - \1@@ER  - /

# Deal with doi and webpages
s/[Dd][Oo][Ii]:\(.*\)@@N1/@@DO  - \1@@N1/
s/\(https*:.*\)@@N1/@@UR  - \1@@N1/
s/\(ftp:.*\)@@N1/@@UR  - \1@@N1/
s/[Rr]etrieved from:*//

# Format Author (Year). Title1
s/^\([^@(]*\) *(\([^)]*\))\.* */@@A1  - \1@@Y1  - \2@@T1  - /

# Format Title (Year)
s/@@A1  - \([^@,]*\),* *\(@@Y1  - .*\)@@T1  - /@@T1  - \1\2@@T2  - /

# Format In Editors, Title2
s/\(@@T1  - [^@]*\)[,.?] In \([^<@]*\)\<i\>/\1@@A2  - \2@@T2  - \<i\>/

# Format (Edition, Volume, Pages).
s/ (\([^@)]*\) ed\.,* */@@ET  - \1(/
s/([Vv]ol\.* \([^@),]*\)/@@VL  - \1(/
s/(,* *[Pp]p*\.* \([^@)]*\)/@@SP  - \1(/
s/()//g

# Format Journal, Vol(Issue), Pages
s/\(@@T1  - [^@<]*\)\.* \<i\>/\1@@JF  - /
s/\(@@JF  - [^@0-9]*\),* \([0-9][0-9-]*[A-Z]*\)/\1@@VL  - \2/
s/\(@@[JV].  - [^@(]*\)(\([0-9-]*\))/\1@@IS  - \2/
s/\(@@[VI].  - [^@,]*\), *\([ 0-9-]*\)/\1@@SP  - \2/

# Format stray pp. Pages
s/ pp*[. ]\(.*@@N1\)/@@SP  - \1/

# Format (Thesis)
s/\(@@T1  - [^@(]*\)(\([^@)]*[Tt]hesis[^@)]*\))/\1@@M3  - \2/
s/\(@@T1  - [^@(]*\)(\([^@)]*[Dd]issertation[^@)]*\))/\1@@M3  - \2/
/@@PB/!s/\(@@M3  - [^@]*\)\. /\1@@PB  - /

# Format Title1. (Report)
s/\(@@T1  - [^@.]*\)\. (*\([^)@]*[Rr]eport[^@)]*\))*/\1@@T2  - \2/
s/\(@@T2  - [^@.]*\)\. (*\([^)@]*[Rr]eport[^@)]*\))*/\1@@T3  - \2/

# Format City: Publisher.
s/\. \([^@:.]*\): \([^@:]*@@N1\)/@@CY  - \1@@PB  - \2/
s/\(@@SP  - [^@.]*\)\. /\1@@PB  - /

# Format Paper presented at
s/\. [ a-zA-Z]* presented at t*h*e* \([^@,]*\), \(.*@@N1\)/@@T2  - \1@@CY  - \2/

# Catch stray publication details
/@@PB/!s/\(@@T1  - [^@.]*\)\. /\1@@PB  - /
s/@@PB  - \([^@:]*\):\([^@]*\)/@@CY  - \1@@PB  - \2/

# Deal with Publisher as Author
s/@@[AT]1  - \([^@]*\)\(.*\)@@PB  - Author/@@A1  - \1\2@@PB  - \1/
/@@T1/!s/@@T2/@@T1/

# Strip italics, bold, underline
: html_loop
s/\<\/*[^>]\>\(.*@@N1\)/\1/
t html_loop

# Deal with extra date info
s/\(@@Y1  - [12][0-9][0-9][0-9]\)[a-z]*\([^@]*\)/\1@@DA  - \2/

# Split Author, I. M. and I. M. Editor at commas etc 
s/\(@@A[12]  - \)\([^@]*\) and /\1\2\1/g
s/\(@@A[12]  - \)\([^@]*\) & /\1\2\1/g
s/\(@@A[12]  - \)\([^@]*\) \.\.\. /\1\2\1et al\1/g
s/\(@@A[12]  - \)\([^@]*\) et al/\1\2\1et al/g
s/\(@@A[12]  - [^@]*\) ([Ee]ds*\.*)/\1/g
: au_loop
s/\(@@A1  - \)\([^@,]*, \)\([^@,]*\), /\1\2\3\1/g
t au_loop
: ed_loop
s/\(@@A2  - \)\([^@,]*\), /\1\2\1/g
t ed_loop
s/@@A2  - \([- .A-Z]*\) \([- 'a-zA-Z]*\)/@@A2  - \2, \1/g

# Tidy up stray punctuation
s/  - [,. ]*/  - /g
s/[, ]*@@/@@/g
s/\([^A-Z]\)\.@@/\1@@/g

# Remove empty fields
s/@@..  - *@@/@@/g
s/@@..  - *@@/@@/g

# Classify records
/ [Pp]resented at /s/^/@@TY  - CPAPER/
/@@TY/!s/^\(.*@@DA\)/@@TY  - NEWS\1/
/@@TY/!s/^\(.*@@JF\)/@@TY  - JOUR\1/
/@@UR/s/@@TY  - JOUR/@@TY  - EJOUR/
/@@TY/!s/^\(.*@@M3\)/@@TY  - THES\1/
/@@TY/!s/^\(.* [Rr]eport\)/@@TY  - RPRT\1/
/@@TY/!s/^\(.*@@UR\)/@@TY  - ELEC\1/
/@@TY/!s/^\(.*@@[AT]2\)/@@TY  - CHAP\1/
/@@TY/!s/^/@@TY  - BOOK/

# Standardise field names
s/@@A1/@@AU/g
s/@@T1/@@TI/
s/@@Y1/@@PY/

# Finally replace placeholder with linebreak.
s/@@/\
/g