# Generic Ref2RIS Converter v. 2.0
# Copyright (C) 2011 Deborah Fitchett
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
########################################################################

# For best results, prepare input according to the ReadMe instructions.
#
# Major known issues - may perform poorly with:
# * authors' full given names
# * journals abbreviated with periods
# * italicised species names
# * citations in all-caps or with no punctuation
# * titles in languages other than English
# * any given style I haven't tested it with...

# Tidy up tags, punctuation, spurious spacing, blank lines, and urls
s/\<[^/>]*\>$//
s/^\<\/[^>]*\>//
s/\(\<[^/>]*\>\)\([^0-9A-Za-z<>]*\)/\2\1/g
s/\([^0-9A-Za-z<>]*\)\(\<\/[^>]*\>\)/\2\1/g
s/\<\([^/>]*\)\>\([^a-zA-Z0-9<>]*\)\<\/\1\>/\2/g
s/\<\/\([^>]*\)\>\([^a-zA-Z0-9<>]*\)\<\1\>/\2/g
s/ *- */-/g
s/\( \)\([ ,]*\)/\1/g
s/^[ 	]*//
s/[ 	]*$//
s/[ 	][ 	]*/ /g
/^$/d
s/\([^/]\)www/\1http:\/\/www/g

# Deliminate records (adding a copy of the full record to the notes field)
s/\(.*\)$/@@XX  - \1@@N1  - \1@@ER  - /

# Strip numbers from start
s/@@XX  - \<sup\>[0-9]*\<\/sup\>[ 	]*/@@XX  - /
s/@@XX  - [[( 	]*[0-9]*[]). 	]*/@@XX  - /

# Find info in square brackets
: info_loop
s/\[\([^]]*\)\]\(.*@@N1\)/@@XX  - \1@@XX  - \2/
t info_loop

# Find doi and webpages
s/[Dd][Oo][Ii]:\([^ <>"]*\)\(.*@@N1\)/@@DO  - \1@@XX  - \2/
s/\(https*:[^ <>"]*\)\(.*@@N1\)/@@UR  - \1@@XX  - \2/
s/\(ftp:[^ <>"]*\)\(.*@@N1\)/@@UR  - \1@@XX  - \2/

# Find labelled page, issue, and volume numbers
s/\(.*\)[^a-zA-Z]pp*[^a-zA-Z0-9]*\([0-9][-0-9]*\)\(.*@@N1\)/\1@@SP  - \2@@XX  - \3/
s/\(.*\)[^a-zA-Z][Nn]os*[^a-zA-Z0-9]*\([0-9][-0-9]*\)\(.*@@N1\)/\1@@IS  - \2@@XX  - \3/
s/\(.*\)[^a-zA-Z][Vv]ols*[^a-zA-Z0-9]*\([0-9][-0-9]*\)\(.*@@N1\)/\1@@VL  - \2@@XX  - \3/

# Start identifying records
/@@DO/s/^/@@TY  - ELEC/
/^@@XX  - .*[Tt]hesis/s/^/@@TY  - THES/
/^@@XX  - .*[Dd]issertation/s/^/@@TY  - THES/
/^@@XX  - .*Conference/s/^/@@TY  - CPAPER/
/^@@XX  - .*Workshop/s/^/@@TY  - CPAPER/
/^@@XX  - .*Congress/s/^/@@TY  - CPAPER/
/^@@XX  - .*Seminar/s/^/@@TY  - CPAPER/
/^@@XX  - .*Symposium/s/^/@@TY  - CPAPER/
/^@@XX  - .*Meeting/s/^/@@TY  - CPAPER/
/^@@XX  - .*Forum/s/^/@@TY  - CPAPER/
/^@@XX  - .*[Rr]eport/s/^/@@TY  - RPRT/
/^@@XX  - .*[Pp]atent/s/^/@@TY  - PAT/
/^@@XX  - .*@@UR/s/^/@@TY  - ELEC/
/@@TY/!s/^/@@TY  - GEN/

# Find identifiable titles
s/\(@@XX  - [^@"]*\)"\([^@"]*\)"\(.*@@N1\)/\1@@T1  - \2@@XX  - \3/
s/\(@@XX  - [^@<]*\)\<[iu]\>\([^@<]*\)\<\/[iu]\>\(.*@@N1\)/\1@@TT  - \2@@XX  - \3/
s/\(@@XX  - [^@<]*\)In:* \(.*@@N1\)/\1@@XX  - \2/

# Split at editor/edition
s/@@XX  - [^a-zA-Z0-9@]*[Ee]ds*[^a-zA-Z0-9@][^a-zA-Z0-9<@]*\(.*@@N1\)/@@XX  - \1/g
s/\([^a-zA-Z0-9@][Ee]ds*[^a-zA-Z0-9@][^a-zA-Z0-9<@]*\)\(.*@@N1\)/\1@@XX  - \2/g

# Rewrite dates
: date_loop
s/\([A-Z][a-z][a-z]\)[a-z]* \([0-9][-0-9]*\),* \([1-2][0-9][0-9][0-9]\)\(.*@@N1\)/\3-\1-\2@@XX  - \4/
s/\([0-9][-0-9]*\)[snrt]*[tdh]* \([A-DF-Z][a-z][a-z]\)[a-z]*,* \([1-2][0-9][0-9][0-9]\)\(.*@@N1\)/\3-\2-\1@@XX  - \4/
s/\([1-2][0-9][0-9][0-9]\)[^-a-zA-Z0-9@]*\([A-Z][a-z][a-z]\)[a-z]*[^-a-zA-Z0-9@]*\([0-9][-0-9]*\)\(.*@@N1\)/\1-\2-\3@@XX  - \4/
t date_loop
s/-Jan-/-01-/g
s/-Feb-/-02-/g
s/-Mar-/-03-/g
s/-Apr-/-04-/g
s/-May-/-05-/g
s/-Jun-/-06-/g
s/-Jul-/-07-/g
s/-Aug-/-08-/g
s/-Sep-/-09-/g
s/-Oct-/-10-/g
s/-Nov-/-11-/g
s/-Dec-/-12-/g
s/\([12][0-9][0-9][0-9]\)-\([0-1][0-9]\)-\([0-9][-0-9]*\)/@@DD  - \1\/\2\/\3\//g
s/\(@@DD  - [12][0-9][0-9][0-9]\/[0-1][0-9]\/\)\([0-9][-/]\)/\10\2/g
s/@@XX  - [^@]*[Aa]ccess[^@]* @@DD  - /@@DA  - /
s/@@XX  - [^@]*[Rr]etriev[^@]* @@DD  - /@@DA  - /
s/@@XX  - [^@]*[Cc]ite[^@]* @@DD  - /@@DA  - /

# Find years
s/\(@@XX  - [^@]*\)\<b\>\([1-2][0-9][0-9][0-9]\)\<\/b\>\(.*@@N1\)/\1@@Y1  - \2@@XX  - \3/
/@@Y1/!s/\(@@XX  - [^@]*\)[[(]\([1-2][0-9][0-9][0-9][^])0-9]*\)[])]\(.*@@N1\)/\1@@Y1  - \2@@XX  - \3/
/@@Y1/!s/\(@@XX  - [^@]*[^-0-9]\)\([1-2][0-9][0-9][0-9]\)\([^-0-9].*@@N1\)/\1@@Y1  - \2@@XX  - \3/
s/\(@@XX  - [^@]*\)n\.d\.\(.*@@N1\)/\1@@Y1  - n\.d\.@@XX  - \2/
s/\(@@XX  - [^@]*\)in press\(.*@@N1\)/\1@@Y1  - in press@@XX  - \2/
/@@Y1/!s/@@DD  - /@@Y1  - /
/@@DD/!s/\(@@TY  - CPAPER.*\)@@Y1/\1@@DA/

# Find unlabelled page numbers, volumes and issues
/@@SP/!s/\(.*[ .:p]\)\([0-9][0-9]*-[0-9][0-9]*\)\([^-0-9].*@N1\)/\1@@SP  - \2@@XX  - \3/
/@@IS/!s/\(.*[^/0-9]\)\([0-9][-0-9]*[A-Z]*\)\<\/[ibu]\>[^a-zA-Z0-9]*\([0-9][-0-9]*\)\([^a-zA-Z0-9]*@@SP\)/\1@@VL  - \2@@IS  - \3\4/
/@@IS/!s/\(.*[^/0-9]\)\([0-9][-0-9]*[A-Z]*\)@@XX  - [^a-zA-Z0-9]*\([0-9][-0-9]*\)\([^a-zA-Z0-9]*@@SP\)/\1@@VL  - \2@@IS  - \3\4/
/@@IS/!s/\(.*[^/0-9]\)\([0-9][-0-9]*[A-Z]*\)[^a-zA-Z0-9][^a-zA-Z0-9]*\([0-9][-0-9]*\)\([^a-zA-Z0-9]*@@SP\)/\1@@VL  - \2@@IS  - \3\4/
/@@VL/!s/\(.*[^/0-9]\)\([0-9][-0-9]*[A-Z]*\)\<\/[ibu]\>\([^a-zA-Z0-9]*@@SP\)/\1@@VL  - \2\3/
/@@VL/!s/\(.*[^/0-9]\)\([0-9][-0-9]*[A-Z]*\)@@XX  - \([^a-zA-Z0-9]*@@SP\)/\1@@VL  - \2\3/
/@@VL/!s/\(.*[^/0-9]\)\([0-9][-0-9]*[A-Z]*\)\([^a-zA-Z0-9]*@@SP\)/\1@@VL  - \2\3/
s/\(@@VL  - [^@]*\)@@XX  - \([^@]*@@SP  - \)/\1@@IS  - \2/
s/@@Y1  - @@VL  - [^a-zA-Z0-9@]*\([1-2][0-9][0-9][0-9]\)[^a-zA-Z0-9@]*@@IS  - /@@Y1  - \1@@VL  - /

# Split at end of titles
s/\. *\. *\./@@Ellipsis/
: split_loop
s/\(@@XX  - [^@]*[^A-Z]\)\. \(.*@@N1\)/\1@@XX  - \2/
t split_loop
s/@@Ellipsis/\.\.\./

# Strip italics, bold, underline
: html_loop
s/\<\/*[^>]\>\(.*@@N1\)/\1/
t html_loop

# Clean up empty fields
s/@@..  - [^a-zA-Z0-9]*@@/@@/g
s/@@..  - [^a-zA-Z0-9]*@@/@@/g

# Identify editions vs editors
s/@@XX  - \([^@]*[^a-zA-Z0-9][^a-zA-Z0-9]*\)[Ee]ditors*\(.*@@N1\)/@@A2  - \1\2/
s/@@XX  - [^a-zA-Z0-9]*\([0-9][^@]*\)[Ee]dition\(.*@@N1\)/@@ET  - \1\2/
s/@@XX  - [^a-zA-Z0-9]*\([0-9][^@]*\)[Ee]d[^a-zA-Z0-9@]*\(.*@@N1\)/@@ET  - \1\2/
s/@@XX  - [^a-zA-Z0-9]*\(Rev[^@]*\)[Ee]dition\(.*@@N1\)/@@ET  - \1\2/
s/@@XX  - [^a-zA-Z0-9]*\(Rev[^@]*\)[Ee]d[^a-zA-Z0-9@]*\(.*@@N1\)/@@ET  - \1\2/
s/@@XX  - \([^@]*[^a-zA-Z0-9][^a-zA-Z0-9]*\)Eds*\([^a-zA-Z0-9].*@@N1\)/@@A2  - \1@@XX  - \2/

# Identify City:Publishing Press (and vice versa)
s/@@XX  - \([^@]*\):\([^@]*Press[^@]*\)/@@CY  - \1@@PB  - \2/
s/@@XX  - \([^@]*\):\([^@]*Publi[sc][ha][^@]*\)/@@CY  - \1@@PB  - \2/
s/@@XX  - \([^@]*Press[^@]*\):\([^@]*\)/@@PB  - \1@@CY  - \2/
s/@@XX  - \([^@]*Publi[sc][ha][^@]*\):\([^@]*\)/@@PB  - \1@@CY  - \2/

# Identify City, State:Publisher (and vice versa)
/@@PB/!s/@@XX  - \([^@]*[A-Z][A-Z][^a-zA-Z0-9@]*\):\([^@]*\)/@@CY  - \1@@PB  - \2/
/@@PB/!s/@@XX  - \([^@]*\):\([^@]*[A-Z][A-Z][^a-zA-Z0-9@]*@@\)/@@PB  - \1@@CY  - \2/

# Identify author fields
s/@@XX  - \([^@]*et al[^@]*\)/@@AA  - \1/
s/@@XX  - \([^@]*, \)\.\.\.\([^@]*\)/@@AA  - \1 et al, \2/
s/@@XX  - \([^@]*&[^@]*\)/@@AA  - \1/
s/@@XX  - \([^@]* [JS]r[^@]*\)/@@AA  - \1/
s/@@XX  - \(['a-zA-Z][-' a-zA-Z]*[, ]*[A-Z][-. A-Z]*[,.;@]\)/@@AA  - \1/
s/@@XX  - \([A-Z][-. A-Z]* [a-zA-Z][-' a-zA-Z]*[,.;@]\)/@@AA  - \1/

# Identify more record types
/@@A2/s/@@TY  - [A-Z]*/@@TY  - CHAP/
/@@VL/s/@@TY  - GEN/@@TY  - JOUR/
/@@VL/s/@@TY  - ELEC/@@TY  - EJOUR/

# Guess at City:Publisher
/@@TY  - E*JOUR/!s/@@XX  - \([^@]*, [^@]*\):\([^@]*\)/@@CY  - \1@@PB  - \2/
/@@TY  - E*JOUR/!s/@@XX  - \([^@]*\):\([^@]*, [^@]*\)/@@PB  - \1@@CY  - \2/

# Identify miscellaneous fields
s/@@XX  - \([^@]*[Tt]hesis[^@]*\)/@@M3  - \1/g
s/@@XX  - \([^@]*[Dd]issertation[^@]*\)/@@M3  - \1/g
s/@@XX  - \([^@]*\) [Pp]atent [^@0-9]*\([0-9]*\)/@@CY  - \1@@SN  - \2/g

# Identify title fields
s/@@XX  - \([^@]* a [^@]*\)/@@TT  - \1/g
s/@@XX  - \([^@]* an [^@]*\)/@@TT  - \1/g
s/@@XX  - \([^@]* in [^@]*\)/@@TT  - \1/g
s/@@XX  - \([^@]* on [^@]*\)/@@TT  - \1/g
s/@@XX  - \([^@]* and [^@]*\)/@@TT  - \1/g
s/@@XX  - \([^@]* for [^@]*\)/@@TT  - \1/g
s/@@XX  - \([^@]*[Rr]eport[^@]*\)/@@TT  - \1/g

# Final guess at Publisher
/@@PB/!s/@@XX  - \([^@]*:[^@]*\)/@@PB  - \1/
/@@PB/s/@@TY  - GEN/@@TY  - BOOK/

# Assign authors
s/@@XX  - \([^@]*@@Y1\)/@@AA  - \1/
/@@A1/!s/@@AA/@@A1/
/@@A2/!s/@@AA/@@A2/
s/@@AA/@@A3/
s/@@AA/@@XX/g

# Split authors
s/\(@@A.  - \)\([^@]*\) et al */\1\2\1et al\1/
: au_loop
s/\(@@A.  - \)\([^@]*\); /\1\2\1/
s/\(@@A.  - \)\([^@]*\)& /\1\2\1/
s/\(@@A.  - \)\(['b-zA-Z][-' a-zA-Z]*\),* \([A-Z][- .A-Z]*\), /\1\2, \3\1/
s/\(@@A.  - \)\([A-Z][- .A-Z]* ['a-zA-Z][-' a-zA-Z]*\), /\1\2\1/
t au_loop
s/\(@@A.  - \)\(['b-zA-Z][-' a-zA-Z]*\),* \([A-Z][- .A-Z]*\)\([A-Z][a-z]\)/\1\2, \3@@TT  - \4/g
s/\(@@A.  -\)\([^@]*\)and /\1\2\1 /g
s/\(@@A.  - \)\([A-Z][- .A-Z]*\) \(['a-zA-Z][-' a-zA-Z]*\)/\1\3, \2/g

# Assign titles
/@@T1/!s/@@TT/@@T1/
/@@T1/!s/@@T2/@@T1/
/@@T2/!s/@@TT/@@T2/
/@@T3/!s/@@TT/@@T3/
s/@@TT/@@XX/

# Assign dates
/@@Y1/!s/@@DD/@@Y1/
/@@Y2/!s/@@DD/@@Y2/
/@@DA/!s/@@DD/@@DA/
s/@@DD/@@XX/g

# Clean punctuation from start and end of fields
s/\(@@..  - \)[^a-zA-Z0-9@<>]*/\1/g
s/[^a-zA-Z0-9<>]*\(@@..  - \)/\1/g

# Clean up empty fields
s/@@..  - [^a-zA-Z0-9]*@@/@@/g
s/@@..  - [^a-zA-Z0-9]*@@/@@/g

# Guess at, or collapse, unidentified fields
s/\(@@TY  - [A-Z]*\)@@XX  - \([^@]*\)\(@@..  - \)/\1\3\2\. /
/@@T1/!s/@@XX  - /@@T1  - /
/@@T2/!s/@@XX  - /@@T2  - /
/@@TY  - E*JOUR/s/@@T2/@@JF/
/@@T3/!s/@@XX  - /@@T3  - /
s/@@XX  - /. /g

# Standardise field names
s/@@A1/@@AU/g
s/@@T1/@@TI/
s/@@Y1/@@PY/

# Finally replace placeholder with linebreak.
s/@@/\
/g